@@ -7,6 +7,7 @@ import xarray.testing as xrt
 import dask.array
 import numpy as np
 from pathlib import Path
+from scipy.integrate import quad_vec
 
 from xbout.tests.test_load import bout_xyt_example_files, create_bout_ds
 from xbout.tests.test_region import (
@@ -17,7 +18,8 @@ from xbout.tests.test_region import (
 )
 from xbout import BoutDatasetAccessor, open_boutdataset
 from xbout.geometries import apply_geometry
-from xbout.utils import _set_attrs_on_all_vars
+from xbout.utils import _set_attrs_on_all_vars, _1d_coord_from_spacing
+from xbout.tests.utils_for_tests import set_geometry_from_input_file
 
 
 EXAMPLE_OPTIONS_FILE_PATH = "./xbout/tests/data/options/BOUT.inp"
@@ -663,7 +665,7 @@ class TestBoutDatasetMethods:
             nt=1,
             guards=guards,
             grid="grid",
-            topology="disconnected-double-null",
+            topology="lower-disconnected-double-null",
         )
 
         ds = open_boutdataset(
@@ -1153,8 +1155,6 @@ class TestBoutDatasetMethods:
                 "G3",
                 "J",
                 "Bxy",
-                "dx",
-                "dy",
             )
         )
 
@@ -1166,7 +1166,6 @@ class TestBoutDatasetMethods:
         # interpolate_parallel are correct.
         # This test does not test the accuracy of the parallel interpolation (there are
         # other tests for that).
-
         # Note using more than MXG x-direction points and MYG y-direction points per
         # output file ensures tests for whether boundary cells are present do not fail
         # when using minimal numbers of processors
@@ -1180,7 +1179,6 @@ class TestBoutDatasetMethods:
             grid="grid",
             topology="limiter",
         )
-
         ds = open_boutdataset(
             datapath=dataset_list,
             gridfilepath=grid_ds,
@@ -1188,34 +1186,396 @@ class TestBoutDatasetMethods:
             keep_xboundaries=True,
             keep_yboundaries=False,
         )
-
         # Get high parallel resolution version of ds, and check that
         ds = ds.bout.interpolate_parallel(["n", "T"])
-
         mxg = 2
         myg = 2
-
         ixs1 = ds.metadata["ixseps1"]
-
         for var in ["n", "T"]:
             v = ds[var]
             v_noregions = v.copy(deep=True)
-
             # Remove attributes that are expected to be different
             del v_noregions.attrs["regions"]
-
             v_core = v.bout.from_region("core", with_guards={"theta": 0})
-
             # Remove attributes that are expected to be different
             del v_core.attrs["regions"]
             xrt.assert_identical(v_noregions.isel(x=slice(ixs1 + mxg)), v_core)
-
             v_sol = v.bout.from_region("SOL")
-
             # Remove attributes that are expected to be different
             del v_sol.attrs["regions"]
             xrt.assert_identical(v_noregions.isel(x=slice(ixs1 - mxg, None)), v_sol)
 
+    def test_integrate_midpoints_slab(self, bout_xyt_example_files):
+        # Create data
+        dataset_list = bout_xyt_example_files(
+            None, lengths=(4, 100, 110, 120), nxpe=1, nype=1, nt=1, syn_data_type=1
+        )
+        ds = open_boutdataset(dataset_list)
+        t = np.linspace(0.0, 8.0, 4)[:, np.newaxis, np.newaxis, np.newaxis]
+        x = np.linspace(0.05, 9.95, 100)[np.newaxis, :, np.newaxis, np.newaxis]
+        y = np.linspace(0.1, 21.9, 110)[np.newaxis, np.newaxis, :, np.newaxis]
+        z = np.linspace(0.15, 35.85, 120)[np.newaxis, np.newaxis, np.newaxis, :]
+        ds["t"].data[...] = t.squeeze()
+        ds["dx"].data[...] = 0.1
+        ds["dy"].data[...] = 0.2
+        ds.metadata["dz"] = 0.3
+        tfunc = 1.5 * t
+        xfunc = x ** 2
+        yfunc = 10.0 * y - y ** 2
+        zfunc = 2.0 * z ** 2 - 30.0 * z
+        ds["n"].data[...] = tfunc * xfunc * yfunc * zfunc
+        tintegral = 48.0
+        xintegral = 1000.0 / 3.0
+        yintegral = 5.0 * 22.0 ** 2 - 22.0 ** 3 / 3.0
+        zintegral = 2.0 * 36.0 ** 3 / 3.0 - 15.0 * 36.0 ** 2
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims="t"),
+            (tintegral * xfunc * yfunc * zfunc).squeeze(),
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims="x"),
+            (tfunc * xintegral * yfunc * zfunc).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims="y"),
+            (tfunc * xfunc * yintegral * zfunc).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims="z"),
+            (tfunc * xfunc * yfunc * zintegral).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["t", "x"]),
+            (tintegral * xintegral * yfunc * zfunc).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["t", "y"]),
+            (tintegral * xfunc * yintegral * zfunc).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["t", "z"]),
+            (tintegral * xfunc * yfunc * zintegral).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["x", "y"]),
+            (tfunc * xintegral * yintegral * zfunc).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["x", "z"]),
+            (tfunc * xintegral * yfunc * zintegral).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["y", "z"]),
+            (tfunc * xfunc * yintegral * zintegral).squeeze(),
+            rtol=1.2e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["t", "x", "y"]),
+            (tintegral * xintegral * yintegral * zfunc).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["t", "x", "z"]),
+            (tintegral * xintegral * yfunc * zintegral).squeeze(),
+            rtol=1.0e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["t", "y", "z"]),
+            (tintegral * xfunc * yintegral * zintegral).squeeze(),
+            rtol=1.2e-4,
+        )
+        # default dims
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n"),
+            (tfunc * xintegral * yintegral * zintegral).squeeze(),
+            rtol=1.4e-4,
+        )
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["t", "x", "y", "z"]),
+            (tintegral * xintegral * yintegral * zintegral),
+            rtol=1.4e-4,
+        )
+
+    @pytest.mark.parametrize(
+        "location", ["CELL_CENTRE", "CELL_XLOW", "CELL_YLOW", "CELL_ZLOW"]
+    )
+    def test_integrate_midpoints_salpha(self, bout_xyt_example_files, location):
+        # Create data
+        nx = 100
+        ny = 110
+        nz = 120
+        dataset_list = bout_xyt_example_files(
+            None,
+            lengths=(4, nx, ny, nz),
+            nxpe=1,
+            nype=1,
+            nt=1,
+            syn_data_type=1,
+            guards={"x": 2, "y": 2},
+        )
+        ds = open_boutdataset(dataset_list)
+
+        ds, options = set_geometry_from_input_file(ds, "s-alpha.inp")
+
+        ds = apply_geometry(ds, "toroidal")
+
+        # Integrate 1 so we just get volume, areas and lengths
+        ds["n"].values[:] = 1.0
+        ds["n"].attrs["cell_location"] = location
+
+        # remove boundary cells (don't want to integrate over those)
+        ds = ds.bout.remove_yboundaries()
+        if ds.metadata["keep_xboundaries"] and ds.metadata["MXG"] > 0:
+            mxg = ds.metadata["MXG"]
+            xslice = slice(mxg, -mxg)
+        else:
+            xslice = slice(None)
+        ds = ds.isel(x=xslice)
+
+        # Test geometry has major radius R and goes between minor radii a-Lr/2 and
+        # a+Lr/2
+        R = options.evaluate_scalar("mesh:R0")
+        a = options.evaluate_scalar("mesh:a")
+        Lr = options.evaluate_scalar("mesh:Lr")
+        rinner = a - Lr / 2.0
+        router = a + Lr / 2.0
+        r = options.evaluate("mesh:r").squeeze()[xslice]
+        if location == "CELL_XLOW":
+            rinner = rinner - Lr / (2.0 * nx)
+            router = router - Lr / (2.0 * nx)
+            r = r - Lr / (2.0 * nx)
+        q = options.evaluate_scalar("mesh:q")
+        T_total = (ds.sizes["t"] - 1) * (ds["t"][1] - ds["t"][0]).values
+
+        # Volume of torus with circular cross-section of major radius R and minor radius
+        # a is 2*pi*R*pi*a^2
+        # https://en.wikipedia.org/wiki/Torus
+        # Default is to integrate over all spatial dimensions
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n"),
+            2.0 * np.pi * R * np.pi * (router ** 2 - rinner ** 2),
+            rtol=1.0e-5,
+            atol=0.0,
+        )
+        # Pass all spatial dims explicitly
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["x", "theta", "zeta"]),
+            2.0 * np.pi * R * np.pi * (router ** 2 - rinner ** 2),
+            rtol=1.0e-5,
+            atol=0.0,
+        )
+        # Integrate in time too
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["t", "x", "theta", "zeta"]),
+            T_total * 2.0 * np.pi * R * np.pi * (router ** 2 - rinner ** 2),
+            rtol=1.0e-5,
+            atol=0.0,
+        )
+        # Integrate in time using dims=...
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=...),
+            T_total * 2.0 * np.pi * R * np.pi * (router ** 2 - rinner ** 2),
+            rtol=1.0e-5,
+            atol=0.0,
+        )
+
+        # Area of torus with circular cross-section of major radius R and minor radius a
+        # is 2*pi*R*2*pi*a
+        # https://en.wikipedia.org/wiki/Torus
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["theta", "zeta"]),
+            (2.0 * np.pi * R * 2.0 * np.pi * r)[np.newaxis, :]
+            * np.ones(ds.sizes["t"])[:, np.newaxis],
+            rtol=1.0e-5,
+            atol=0.0,
+        )
+        # Integrate in time too
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["t", "theta", "zeta"]),
+            T_total * 2.0 * np.pi * R * 2.0 * np.pi * r,
+            rtol=1.0e-5,
+            atol=0.0,
+        )
+
+        # Area of cross section in poloidal plane is difference of circle with radius
+        # router and circle with radius rinner, pi*(router**2 - rinner**2)
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["x", "theta"]),
+            np.pi * (router ** 2 - rinner ** 2),
+            rtol=1.0e-5,
+            atol=0.0,
+        )
+        # Integrate in time too
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["t", "x", "theta"]),
+            T_total * np.pi * (router ** 2 - rinner ** 2),
+            rtol=1.0e-5,
+            atol=0.0,
+        )
+
+        # x-z planes are 'conical frustrums', with area pi*(Rinner + Router)*Lr
+        # https://en.wikipedia.org/wiki/Frustum
+        theta = _1d_coord_from_spacing(ds["dy"], "theta").values
+        if location == "CELL_YLOW":
+            theta = theta - 2.0 * np.pi / (2.0 * ny)
+        Rinner = R + rinner * np.cos(theta)
+        Router = R + router * np.cos(theta)
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["x", "zeta"]),
+            (np.pi * (Rinner + Router) * Lr)[np.newaxis, :]
+            * np.ones(ds.sizes["t"])[:, np.newaxis],
+            rtol=1.0e-5,
+            atol=0.0,
+        )
+        # Integrate in time too
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["t", "x", "zeta"]),
+            T_total * (np.pi * (Rinner + Router) * Lr),
+            rtol=1.0e-5,
+            atol=0.0,
+        )
+
+        # Radial lines have length Lr
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["x"]),
+            Lr,
+            rtol=1.0e-5,
+            atol=0.0,
+        )
+        # Integrate in time too
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["t", "x"]),
+            T_total * Lr,
+            rtol=1.0e-5,
+            atol=0.0,
+        )
+
+        # Poloidal lines have length 2*pi*r
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["theta"]),
+            (2.0 * np.pi * r)[np.newaxis, :, np.newaxis]
+            * np.ones(ds.sizes["t"])[:, np.newaxis, np.newaxis]
+            * np.ones(ds.sizes["zeta"])[np.newaxis, np.newaxis, :],
+            rtol=1.0e-5,
+            atol=0.0,
+        )
+        # Integrate in time too
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["t", "theta"]),
+            T_total
+            * (2.0 * np.pi * r)[:, np.newaxis]
+            * np.ones(ds.sizes["zeta"])[np.newaxis, :],
+            rtol=1.0e-5,
+            atol=0.0,
+        )
+
+        # Field line length
+        # -----------------
+        #
+        # Field lines, parameterised by poloidal angle theta, have (R, Z, zeta)
+        # coordinates
+        # X = (R + r*cos(theta),
+        #      -r*sin(theta),
+        #      -q*2*atan(sqrt((1-r/R0)/(1+r/R0))*tan(theta/2))
+        #     )
+        # using d(arctan(x))/dx = 1/(1 + x**2) and d(tan(x))/dx = 1/cos(x)**2
+        # dX/dtheta = (r*sin(theta),
+        #              + r*cos(theta),
+        #              - q * 2
+        #                * sqrt((1-r/R0)/(1+r/R0))/(2*cos(theta/2)**2)
+        #                / (1 + (1-r/R0)/(1+r/R0)*tan(theta/2)**2)
+        #             )
+        #           = (r*sin(theta),
+        #              + r*cos(theta),
+        #              - q
+        #                * sqrt((1-r/R0)/(1+r/R0))/cos(theta/2)**2
+        #                / (1 + (1-r/R0)/(1+r/R0)*tan(theta/2)**2)
+        #             )
+        #           = (r*sin(theta),
+        #              + r*cos(theta),
+        #              - q
+        #                * sqrt((1-r/R0)/(1+r/R0))
+        #                / (cos(theta/2)**2 + (1-r/R0)/(1+r/R0)*sin(theta/2)**2)
+        #             )
+        # Line element dl = |dR + dZ + R dzeta|
+        #                 = |dR + dZ + (R0 + r cos(theta)) dzeta|
+        #                 = |dX/dtheta|*dtheta
+        # |dX/dtheta| = |
+        #             = sqrt(
+        #                r**2*sin(theta)**2
+        #                + r**2*cos(theta)**2
+        #                + (R0 + r cos(theta))**2 * q**2
+        #                  * (1-r/R0)/(1+r/R0)
+        #                  / (cos(theta/2)**2 + (1-r/R0)/(1+r/R0)*sin(theta/2)**2)**2
+        #               )
+        #             = sqrt(
+        #                r**2
+        #                + (R0 + r cos(theta))**2 * q**2
+        #                  * (1-r/R0)/(1+r/R0)
+        #                  / (cos(theta/2)**2 + (1-r/R0)/(1+r/R0)*sin(theta/2)**2)**2
+        #               )
+        # field line length is int_{0}^{2 pi} dl
+        a = r ** 2
+        c = (1 - r / R) / (1 + r / R)
+        b = q ** 2 * c
+
+        def func(theta):
+            return np.sqrt(
+                a
+                + b
+                * (R + r * np.cos(theta)) ** 2
+                / (np.cos(theta / 2.0) ** 2 + c * np.sin(theta / 2.0) ** 2) ** 2
+            )
+
+        integral, _ = quad_vec(func, 0.0, 2.0 * np.pi)
+
+        ds["n_aligned"] = ds["n"].bout.to_field_aligned()
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n_aligned", dims=["theta"]),
+            integral[np.newaxis, :, np.newaxis]
+            * np.ones(ds.sizes["t"])[:, np.newaxis, np.newaxis]
+            * np.ones(ds.sizes["zeta"])[np.newaxis, np.newaxis, :],
+            rtol=1.0e-5,
+            atol=0.0,
+        )
+        # Integrate in time too
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n_aligned", dims=["t", "theta"]),
+            T_total
+            * integral[:, np.newaxis]
+            * np.ones(ds.sizes["zeta"])[np.newaxis, :],
+            rtol=1.0e-5,
+            atol=0.0,
+        )
+
+        # Toroidal lines have length 2*pi*Rxy
+        if location == "CELL_CENTRE":
+            R_2d = ds["R"]
+        else:
+            R_2d = ds[f"Rxy_{location}"]
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["zeta"]),
+            (2.0 * np.pi * R_2d).values[np.newaxis, :, :]
+            * np.ones(ds.sizes["t"])[:, np.newaxis, np.newaxis],
+            rtol=1.0e-5,
+            atol=0.0,
+        )
+        # Integrate in time too
+        npt.assert_allclose(
+            ds.bout.integrate_midpoints("n", dims=["t", "zeta"]),
+            T_total * (2.0 * np.pi * R_2d),
+            rtol=1.0e-5,
+            atol=0.0,
+        )
+
     def test_interpolate_from_unstructured(self, bout_xyt_example_files):
         dataset_list, grid_ds = bout_xyt_example_files(
             None,
@@ -1224,7 +1584,7 @@ class TestBoutDatasetMethods:
             nype=6,
             nt=1,
             grid="grid",
-            topology="disconnected-double-null",
+            topology="upper-disconnected-double-null",
         )
 
         ds = open_boutdataset(
@@ -1275,7 +1635,7 @@ class TestBoutDatasetMethods:
             nype=6,
             nt=1,
             grid="grid",
-            topology="disconnected-double-null",
+            topology="lower-disconnected-double-null",
         )
 
         ds = open_boutdataset(
@@ -1735,7 +2095,7 @@ class TestSaveRestart:
             nt=1,
             guards={"x": 2, "y": 2},
             lengths=(6, 5, 4, 7),
-            topology="disconnected-double-null",
+            topology="upper-disconnected-double-null",
             write_to_disk=True,
         )
 
@@ -1800,7 +2160,7 @@ class TestSaveRestart:
             nt=1,
             guards={"x": 2, "y": 2},
             lengths=(6, 5, 4, 7),
-            topology="disconnected-double-null",
+            topology="lower-disconnected-double-null",
             write_to_disk=True,
         )
 
